from django.core.exceptions import PermissionDenied

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from geokey.core.decorators import handle_exceptions_for_ajax
from geokey.core.exceptions import MalformedRequestData
from geokey.users.models import User
from .base import SingleAllContribution
from ..models import Comment
from ..serializers import CommentSerializer


class CommentAbstractAPIView(APIView):
    """
    Abstract class for Comments views
    """
    def get_list_and_respond(self, user, observation):
        """
        Reponsds to a GET request with a list of all comments for an
        observation

        Parameters
        ----------
        user : geokey.users.models.User
            User authenticated with the request
        observation : geokey.contributions.models.Observation
            Observation the comments are requested for

        Returns
        -------
        rest_framework.response.Respones
            Contains the serialised comments
        """
        comments = observation.comments.filter(respondsto=None)
        serializer = CommentSerializer(
            comments, many=True, context={'user': user})
        return Response(serializer.data, status=status.HTTP_200_OK)

    def create_and_respond(self, request, observation):
        """
        Reponsds to a POST request by creating a comment

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        observation : geokey.contributions.models.Observation
            Observation the comments are requested for

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comment
        """
        if observation.status == 'draft':
            raise MalformedRequestData(
                'This contribution is a draft. You may not comment on drafts.'
            )

        user = request.user
        if user.is_anonymous():
            user = User.objects.get(display_name='AnonymousUser')

        respondsto = request.DATA.get('respondsto') or None
        if respondsto is not None:
            try:
                respondsto = observation.comments.get(pk=respondsto)
            except Comment.DoesNotExist:
                raise MalformedRequestData('The comment you try to respond to'
                                           ' is not a comment to the '
                                           'observation.')

        review_status = request.DATA.get('review_status') or None
        if review_status == 'open' and observation.status != 'review':
            observation.update(None, user, status='review')

        comment = Comment.objects.create(
            text=request.DATA.get('text'),
            respondsto=respondsto,
            commentto=observation,
            creator=user,
            review_status=review_status
        )

        serializer = CommentSerializer(comment, context={'user': request.user})
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    def update_and_respond(self, request, comment):
        """
        Reponsds to a PATCH request by updating a comment

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        comment : geokey.contributions.models.Comment
            Comment to be updated

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comment

        Raises
        ------
        PermissionDenied
            if the user authenticated with the request is not allowed to
            update the comment
        """
        user = request.user
        data = request.DATA

        can_moderate = comment.commentto.project.can_moderate(request.user)
        is_owner = comment.creator == request.user

        if (is_owner or can_moderate):
            if data.get('review_status') == 'resolved' and not can_moderate:
                raise PermissionDenied('You are not a project moderator and '
                                       'therefore not eligable to resolve this'
                                       ' comment.')

            serializer = CommentSerializer(
                comment,
                data=data,
                partial=True,
                context={'user': user}
            )

            if serializer.is_valid():
                serializer.save()

                if (not comment.commentto.comments.filter(
                        review_status='open').exists()):
                    comment.commentto.update(None, user, status='active')

                return Response(serializer.data)
            else:
                return Response(
                    serializer.errors,
                    status=status.HTTP_400_BAD_REQUEST
                )

        else:
            raise PermissionDenied('You are neither the author if this comment'
                                   ' nor a project moderator and therefore'
                                   ' not eligable to edit this comment.')

    def delete_and_respond(self, request, comment):
        """
        Reponsds to a DELETE request by deleting a comment

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        comment : geokey.contributions.models.Comment
            Comment to be updated

        Returns
        -------
        rest_framework.response.Respone
            Empty response indicating success

        Raises
        ------
        PermissionDenied
            if the user authenticated with the request is not allowed to
            delete the comment
        """
        observation = comment.commentto
        if (comment.creator == request.user or
                observation.project.can_moderate(request.user)):

            comment.delete()

            if not observation.comments.filter(review_status='open').exists():
                observation.update(None, request.user, status='active')

            return Response(status=status.HTTP_204_NO_CONTENT)
        else:
            raise PermissionDenied('You are neither the author if this comment'
                                   ' nor a project moderator and therefore'
                                   ' not eligable to delete this comment.')


class AllContributionsCommentsAPIView(
        CommentAbstractAPIView, SingleAllContribution):

    @handle_exceptions_for_ajax
    def get(self, request, project_id, observation_id):
        """
        Returns a list of all comments of the observation

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comments
        """
        observation = self.get_object(request.user, project_id, observation_id)
        return self.get_list_and_respond(request.user, observation)

    @handle_exceptions_for_ajax
    def post(self, request, project_id, observation_id):
        """
        Adds a new comment to the observation

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comment
        """
        observation = self.get_object(request.user, project_id, observation_id)
        return self.create_and_respond(request, observation)


class AllContributionsSingleCommentAPIView(
        CommentAbstractAPIView, SingleAllContribution):

    @handle_exceptions_for_ajax
    def patch(self, request, project_id, observation_id, comment_id):
        """
        Updates a comment

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comment
        """
        observation = self.get_object(request.user, project_id, observation_id)
        comment = observation.comments.get(pk=comment_id)
        return self.update_and_respond(request, comment)

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, observation_id, comment_id):
        """
        Deletes a comment from the observation

        Parameters
        ----------
        request : rest_framework.request.Request
            User authenticated with the request
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        rest_framework.response.Respone
            Contains the serialised comment
        """
        observation = self.get_object(request.user, project_id, observation_id)
        comment = observation.comments.get(pk=comment_id)
        return self.delete_and_respond(request, comment)



import datetime
import factory

from geokey.users.tests.model_factories import UserF
from geokey.projects.tests.model_factories import ProjectF
from geokey.categories.tests.model_factories import CategoryFactory

from ..models import Location, Observation, Comment


class LocationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Location

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    geometry = 'POINT(-0.134040713310241 51.52447878755655)'
    created_at = datetime.date(2014, 11, 11)
    creator = factory.SubFactory(UserF)
    status = 'active'
    version = 1
    private = False
    private_for_project = None


class ObservationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Observation

    location = factory.SubFactory(LocationFactory)
    project = factory.SubFactory(ProjectF)
    status = 'active'
    category = factory.SubFactory(CategoryFactory)
    created_at = datetime.date(2014, 11, 11)
    creator = factory.SubFactory(UserF)
    version = 1


class CommentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Comment

    text = factory.Sequence(lambda n: 'Comment number %d' % n)
    created_at = datetime.date(2014, 11, 11)
    creator = factory.SubFactory(UserF)
    commentto = factory.SubFactory(ObservationFactory)
    respondsto = None
    status = 'active'



from django.db import models
from django.core.exceptions import PermissionDenied

from model_utils.managers import InheritanceManager

from geokey.projects.models import Project

from .base import STATUS


class ActiveMixin(object):
    """
    Mixin to provide a queryset method filtering for all instances of the
    model that have status active.
    """
    def active(self):
        """
        Returns all active instances

        Returns
        -------
        django.db.models.Queryset
            All active instances
        """
        return self.get_queryset().filter(status=STATUS.active)


class CategoryManager(ActiveMixin, models.Manager):
    """
    Adds category-specific manager methods.
    """
    def get_queryset(self):
        """
        Returns all categories

        Returns
        -------
        django.db.models.Queryset
            All categories, excluding deleted
        """
        return super(
            CategoryManager,
            self
        ).get_queryset().exclude(status=STATUS.deleted)

    def get_list(self, user, project_id):
        """
        Returns all category objects the user is allowed to access. Project
        admins can access all categories, other users will get access to
        active categories only.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database

        Returns
        -------
        django.db.models.Queryset
            Queryset of all categories the user is allowed to access
        """
        project = Project.objects.get_single(user, project_id)
        if (project.is_admin(user)):
            return project.categories.all()
        else:
            return project.categories.active()

    def get_single(self, user, project_id, category_id):
        """
        Returns all a single category.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database
        category_id : int
            ID identifying the category in the database

        Returns
        -------
        geokey.categories.models.Category

        Raises
        ------
        PermissionDenied
            if the user is not eligable to access the category
        """
        category = Project.objects.get_single(
            user, project_id).categories.get(pk=category_id)

        if (category.status == STATUS.active or
                category.project.is_admin(user)):
            return category
        else:
            raise PermissionDenied('You are not allowed to access this '
                                   'category')

    def as_admin(self, user, project_id, category_id):
        """
        Returns all a single category for a project admin.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database
        category_id : int
            ID identifying the category in the database

        Returns
        -------
        geokey.categories.models.Category

        Raises
        ------
        PermissionDenied
            if the user is not an admin of the project
        """
        return Project.objects.as_admin(
            user, project_id
            ).categories.get(pk=category_id)

    def create(self, *args, **kwargs):
        """
        Creates a new category. Overwrites method to set the order of the
        category in the list of categories.

        Returns
        -------
        geokey.categories.models.Category
            The newly created category
        """
        category = super(CategoryManager, self).create(*args, **kwargs)

        category.order = category.project.categories.count() - 1
        category.save()

        return category


class FieldManager(ActiveMixin, InheritanceManager):
    """
    Adds field-specific manager methods.
    """
    use_for_related_fields = True

    def get_queryset(self):
        """
        Returns all field subclasses ordered by `order` within the category.

        Returns
        -------
        django.db.models.Queryset
            All fields, excluding deleted
        """
        return super(FieldManager, self).get_queryset().order_by(
            'order').select_subclasses()

    def get_list(self, user, project_id, category_id):
        """
        Returns all fields the user is allowed to access.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database
        category_id : int
            ID identifying the category in the database

        Returns
        -------
        django.db.models.Queryset
        """
        project = Project.objects.get_single(user, project_id)

        if project.is_admin(user):
            return project.categories.get(
                pk=category_id).fields.all().select_subclasses()
        else:
            category = project.categories.get(
                pk=category_id)
            if category.status == STATUS.active:
                return category.fields.active().select_subclasses()

    def get_single(self, user, project_id, category_id, field_id):
        """
        Returns all a single field.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database
        category_id : int
            ID identifying the category in the database
        field_id : int
            ID identifying the field in the database

        Returns
        -------
        geokey.categories.models.Field sublcass
            i.e. if you query for a TextField instance
            geokey.categories.models.TextField is return

        Raises
        ------
        PermissionDenied
            if the user is not eligable to access the field
        """
        project = Project.objects.get_single(user, project_id)
        category = project.categories.get(pk=category_id)
        field = category.fields.get_subclass(pk=field_id)

        if project.is_admin(user):
            return field
        else:
            if category.status == STATUS.active:
                if field.status == STATUS.active:
                    return field
                else:
                    raise PermissionDenied('You are not allowed to access '
                                           'this field')
            else:
                raise PermissionDenied('You are not allowed to access this '
                                       'category')

    def as_admin(self, user, project_id, category_id, field_id):
        """
        Returns all a single field for a project admin.

        Parameters
        ----------
        user : geokey.users.models.User
            User the categories are queried for
        project_id : int
            ID identifying the project in the database
        category_id : int
            ID identifying the category in the database
        field_id : int
            ID identifying the field in the database

        Returns
        -------
        geokey.categories.models.Field sublcass
            i.e. if you query for a TextField instance
            geokey.categories.models.TextField is return

        Raises
        ------
        PermissionDenied
            if the user is not admin of the project
        """
        return Project.objects.as_admin(
            user, project_id).categories.get(
            pk=category_id).fields.get_subclass(pk=field_id)


class LookupQuerySet(models.query.QuerySet):
    """
    QuerySet for models having a field status. User by ActiveManager.
    """
    def active(self):
        """
        Returns all active instances

        Returns
        -------
        Queryset of all active instances
        """
        return self.filter(status=STATUS.active)


class LookupValueManager(models.Manager):
    """
    Manager for models having a field status. Is required to render active
    items only in templates.
    """
    use_for_related_fields = True

    def get_queryset(self):
        """
        Returns the Queryset

        Returns
        -------
        django.db.models.Queryset
            all lookupvalues
        """
        return LookupQuerySet(self.model)

    def active(self, *args, **kwargs):
        """
        Returns all active instances

        Returns
        -------
        django.db.models.Queryset
            All active instances
        """
        return self.get_queryset().active(*args, **kwargs)



import os
import glob

from PIL import Image
from StringIO import StringIO

from django.core.files.base import ContentFile
from django.test import TestCase
from django.conf import settings

from nose.tools import raises

from geokey.core.exceptions import FileTypeError
from geokey.contributions.models import MediaFile

from geokey.contributions.tests.model_factories import ObservationFactory
from geokey.users.tests.model_factories import UserF
from .model_factories import ImageFileFactory, get_image


class ModelManagerTest(TestCase):
    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def test_get_queryset(self):
        ImageFileFactory.create_batch(3)
        files = MediaFile.objects.all()

        self.assertEqual(len(files), 3)
        for f in files:
            self.assertEqual('ImageFile', f.type_name)

    def test_create_image(self):
        image_file = MediaFile.objects.create(
            name='Test name',
            description='Test Description',
            contribution=ObservationFactory.create(),
            creator=UserF.create(),
            the_file=get_image()
        )

        self.assertIsNotNone(image_file.image)
        self.assertEqual(image_file.type_name, 'ImageFile')

    @raises(FileTypeError)
    def test_create_not_supported(self):
        xyz_file = StringIO()
        xyz = Image.new('RGBA', size=(50, 50), color=(256, 0, 0))
        xyz.save(xyz_file, 'png')
        xyz_file.seek(0)

        the_file = ContentFile(xyz_file.read(), 'test.xyz')
        the_file.content_type = 'chemical/x-xyz'

        MediaFile.objects.create(
            name='Test name',
            description='Test Description',
            contribution=ObservationFactory.create(),
            creator=UserF.create(),
            the_file=the_file
        )

    # def test_upload_youtube(self):
    #     from geokey.contributions.manager import MediaFileManager
    #     path = '/home/oroick/opencomap/contributions/tests/media/video.MOV'
    #     video_id, swf_link = MediaFileManager()._upload_to_youtube(
    #         'Test video',
    #         path
    #     )
    #     self.assertIsNotNone(video_id)
    #     self.assertIsNotNone(swf_link)



from django import forms

from .models import Project


class ProjectCreateForm(forms.ModelForm):
    """
    Validates the inputs against the model definition.
    Used in .views.ProjectAdminCreateView
    """
    class Meta:
        model = Project
        fields = ('name', 'description', 'isprivate', 'everyone_contributes')



import factory

from geokey.projects.tests.model_factories import ProjectF
from geokey.users.tests.model_factories import UserF

from ..models import (
    Category, TextField, NumericField, DateTimeField, DateField, TimeField,
    LookupField, LookupValue, Field, MultipleLookupField, MultipleLookupValue
)


class CategoryFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Category

    creator = factory.SubFactory(UserF)
    name = factory.Sequence(lambda n: "category %s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    project = factory.SubFactory(ProjectF)
    status = 'active'


class FieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Field

    name = factory.Sequence(lambda n: "field %s" % n)
    key = factory.Sequence(lambda n: "field_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False
    order = 0


class TextFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TextField

    name = factory.Sequence(lambda n: "textfield %s" % n)
    key = factory.Sequence(lambda n: "textfield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class NumericFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = NumericField

    name = factory.Sequence(lambda n: "numericfield %s" % n)
    key = factory.Sequence(lambda n: "numericfield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class DateTimeFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DateTimeField

    name = factory.Sequence(lambda n: "datetimefield %s" % n)
    key = factory.Sequence(lambda n: "datetimefield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class DateFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DateField

    name = factory.Sequence(lambda n: "datefield %s" % n)
    key = factory.Sequence(lambda n: "datefield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class TimeFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TimeField

    name = factory.Sequence(lambda n: "timefield %s" % n)
    key = factory.Sequence(lambda n: "timefield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class LookupFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = LookupField

    name = factory.Sequence(lambda n: "lookupfield %s" % n)
    key = factory.Sequence(lambda n: "lookupfield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class LookupValueFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = LookupValue

    name = factory.Sequence(lambda n: "lookupfield %s" % n)
    field = factory.SubFactory(LookupFieldFactory)
    status = 'active'


class MultipleLookupFieldFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MultipleLookupField

    name = factory.Sequence(lambda n: "lookupfield %s" % n)
    key = factory.Sequence(lambda n: "lookupfield_%s" % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    category = factory.SubFactory(CategoryFactory)
    status = 'active'
    required = False


class MultipleLookupValueFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MultipleLookupValue

    name = factory.Sequence(lambda n: "lookupfield %s" % n)
    field = factory.SubFactory(MultipleLookupFieldFactory)
    status = 'active'



#!/usr/bin/env python

from setuptools import setup, find_packages
from geokey.version import get_version


def get_install_requires():
    """
    parse requirements.txt, ignore links, exclude comments
    """
    requirements = list()
    for line in open('requirements.txt').readlines():
        # skip to next iteration if comment or empty line
        if line.startswith('#') or line.startswith('git+https') or line == '':
            continue
        # add line to requirements
        requirements.append(line.rstrip())

    return requirements

setup(
    name="geokey",
    version=get_version(),
    license="Apache 2.0",
    description="Open-source participatory mapping framework",
    url='http://geokey.org.uk',
    download_url='https://github.com/excites/geokey/releases',
    author="Oliver Roick",
    author_email="o.roick@ucl.ac.uk",
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=get_install_requires(),
    include_package_data=True,
)



from django.test import TestCase
from django.views.generic import View
from django.views.generic.base import TemplateResponseMixin
from rest_framework.test import APIRequestFactory

from geokey.users.tests.model_factories import UserF

from ..views import SuperuserMixin


class ExampleView(SuperuserMixin, TemplateResponseMixin, View):
    template_name = 'base.html'

    def get(self, request):
        return self.render_to_response({
            'success': 'yes'
        })


class SuperuserMixinTest(TestCase):
    def test_user(self):
        view = ExampleView.as_view()
        request = APIRequestFactory().get('http://example.com')
        request.user = UserF.create()
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'This extension is for super users only.'
        )

    def test_superuser(self):
        view = ExampleView.as_view()
        request = APIRequestFactory().get('http://example.com')
        request.user = UserF.create(**{'is_superuser': True})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(
            response,
            'This extension is for super users only.'
        )



from django.test import TestCase
from django.contrib.auth.models import AnonymousUser

from geokey.users.tests.model_factories import UserF
from geokey.contributions.tests.model_factories import ObservationFactory

from .model_factories import ProjectF
from ..serializers import ProjectSerializer


class SerializerTest(TestCase):
    def test_get_geographic_extent(self):
        project = ProjectF.create(**{'geographic_extend': None})
        serializer = ProjectSerializer(project)
        self.assertIsNone(serializer.get_geographic_extent(project))

        project = ProjectF.create()
        serializer = ProjectSerializer(project)
        self.assertIsNotNone(serializer.get_geographic_extent(project))

    def test_get_user_contributions(self):
        user = UserF.create()
        project = ProjectF.create()
        ObservationFactory.create_batch(
            5,
            **{'creator': user, 'project': project}
        )
        serializer = ProjectSerializer(project, context={'user': user})
        self.assertEqual(5, serializer.get_user_contributions(project))

        serializer = ProjectSerializer(
            project, context={'user': AnonymousUser()}
        )
        self.assertEqual(0, serializer.get_user_contributions(project))



from geokey.projects.models import Project
from geokey.contributions.models import Observation


class SingleAllContribution(object):
    """
    Base class for single contributions on the all contributions endpoints
    """
    def get_object(self, user, project_id, observation_id):
        """
        Returns a single Obervation

        Parameters
        ----------
        user : geokey.users.models.User
            User requesting the contribution
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        geokey.contributions.models.Observation

        Raises
        ------
        Observation.DoesNotExist
            If the observations was not found or is not accessible by the user
        """
        project = Project.objects.get_single(user, project_id)

        if project.can_moderate(user):
            return project.get_all_contributions(
                user).for_moderator(user).get(pk=observation_id)
        else:
            return project.get_all_contributions(
                user).for_viewer(user).get(pk=observation_id)


class SingleMyContribution(object):
    """
    Base class for single contributions on the user contributions endpoints
    """
    def get_object(self, user, project_id, observation_id):
        """
        Returns a single Obervation

        Parameters
        ----------
        user : geokey.users.models.User
            User requesting the contribution
        project_id : int
            identifies the project in the data base
        observation_id : int
            identifies the observation in the data base

        Returns
        -------
        geokey.contributions.models.Observation

        Raises
        ------
        Observation.DoesNotExist
            If the observations was not found or is not accessible by the user
        """
        observation = Project.objects.get_single(
            user, project_id).observations.get(pk=observation_id)

        if observation.creator == user:
            return observation
        else:
            raise Observation.DoesNotExist('You are not the creator of this '
                                           'contribution or the contribution '
                                           'has been deleted.')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='creator',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='category',
            name='project',
            field=models.ForeignKey(related_name='categories', to='projects.Project'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(default=django.utils.timezone.now, verbose_name='last login')),
                ('email', models.EmailField(unique=True, max_length=75)),
                ('display_name', models.CharField(unique=True, max_length=50)),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now)),
                ('is_active', models.BooleanField(default=True)),
                ('is_superuser', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField(null=True, blank=True)),
                ('can_contribute', models.BooleanField(default=True)),
                ('can_moderate', models.BooleanField(default=False)),
                ('project', models.ForeignKey(related_name='usergroups', to='projects.Project')),
                ('users', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



import json

from django.test import TestCase
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.core.exceptions import PermissionDenied

from nose.tools import raises

from rest_framework.test import APITestCase
from rest_framework.test import APIRequestFactory, force_authenticate
from rest_framework import status
from rest_framework.renderers import JSONRenderer

from geokey.projects.tests.model_factories import UserF, ProjectF
from geokey.projects.models import Project
from geokey.contributions.models import Comment, Observation

from geokey.users.tests.model_factories import UserGroupF
from ..model_factories import ObservationFactory, CommentFactory

from geokey.contributions.views.comments import (
    AllContributionsSingleCommentAPIView,
    AllContributionsCommentsAPIView,
    CommentAbstractAPIView
)


class CommentAbstractAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.moderator = UserF.create()
        self.commenter = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator, self.commenter]
        )
        self.moderators = UserGroupF(add_users=[self.moderator], **{
            'project': self.project,
            'can_moderate': True
        })
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.creator,
            'status': 'active'
        })
        self.comment = CommentFactory.create(**{
            'commentto': self.observation,
            'creator': self.commenter
        })

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_create_comment_with_admin(self):
        url = reverse('api:project_comments', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.post(url, {'text': 'Comment'})
        request.user = self.admin
        request.DATA = {'text': 'Comment'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.create_and_respond(request, self.observation)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Comment')

    def test_create_reviewcomment_with_admin(self):
        url = reverse('api:project_comments', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.post(
            url, {'text': 'Comment', 'review_status': 'open'}
        )
        request.user = self.admin
        request.DATA = {'text': 'Comment', 'review_status': 'open'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.create_and_respond(request, self.observation)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Comment')
        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'review')

    def test_create_reviewcomment_to_empty_obs_with_admin(self):
        self.observation.properties = None
        self.observation.save()

        url = reverse('api:project_comments', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id
        })
        request = self.factory.post(
            url, {'text': 'Comment', 'review_status': 'open'}
        )
        request.user = self.admin
        request.DATA = {'text': 'Comment', 'review_status': 'open'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.create_and_respond(request, self.observation)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Comment')
        ref = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(ref.status, 'review')

    def test_update_comment_with_admin(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.admin
        request.DATA = {'text': 'Updated'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Updated')
        self.assertEqual(
            Comment.objects.get(pk=self.comment.id).text,
            'Updated'
        )

    def test_update_comment_with_commenter(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.commenter
        request.DATA = {'text': 'Updated'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Updated')
        self.assertEqual(
            Comment.objects.get(pk=self.comment.id).text,
            'Updated'
        )

    def test_update_comment_with_moderator(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.moderator
        request.DATA = {'text': 'Updated'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(json.loads(response.content).get('text'), 'Updated')
        self.assertEqual(
            Comment.objects.get(pk=self.comment.id).text,
            'Updated'
        )

    @raises(PermissionDenied)
    def test_update_comment_with_creator(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.creator
        request.DATA = {'text': 'Updated'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(
            json.loads(response.content).get('text'),
            self.comment.text
        )
        self.assertEqual(
            Comment.objects.get(pk=self.comment.id).text,
            self.comment.text
        )

    def test_update_invalid_comment(self):
        self.project.isprivate = False
        self.project.save()

        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(
            url, {'text': 'Updated', 'review_status': 'blah'}
        )
        force_authenticate(request, user=self.commenter)

        view = AllContributionsSingleCommentAPIView.as_view()
        response = view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id,
            comment_id=self.comment.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class CommentAbstractAPIViewResolveTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.moderator = UserF.create()
        self.commenter = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator, self.commenter]
        )
        self.moderators = UserGroupF(add_users=[self.moderator], **{
            'project': self.project,
            'can_moderate': True
        })
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.creator,
            'status': 'review'
        })
        self.comment = CommentFactory.create(**{
            'commentto': self.observation,
            'creator': self.commenter,
            'review_status': 'open'
        })

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_resolve_comment_with_admin(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.admin
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(
            json.loads(response.content).get('review_status'),
            'resolved'
        )

        reference = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(reference.status, 'active')
        self.assertIsNotNone(reference.properties)

    def test_resolve_comment_with_invalid_review_status(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.admin
        request.DATA = {'review_status': 'closed'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )

        ref = Comment.objects.get(pk=self.comment.id)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(ref.review_status, 'open')

    def test_resolve_one_of_two_comment_with_admin(self):
        CommentFactory.create(**{
            'commentto': self.observation,
            'creator': self.creator,
            'review_status': 'open'
        })

        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.admin
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(
            json.loads(response.content).get('review_status'),
            'resolved'
        )
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'review'
        )

    def test_resolve_comment_with_moderator(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.moderator
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        response = self.render(
            view.update_and_respond(request, self.comment)
        )
        self.assertEqual(
            json.loads(response.content).get('review_status'),
            'resolved'
        )
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    @raises(PermissionDenied)
    def test_resolve_comment_with_creator(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.creator
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        view.update_and_respond(request, self.comment)

    @raises(PermissionDenied)
    def test_resolve_comment_with_commenter(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = self.commenter
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        view.update_and_respond(request, self.comment)

    @raises(PermissionDenied)
    def test_resolve_comment_with_anonymous(self):
        url = reverse('api:project_single_comment', kwargs={
            'project_id': self.project.id,
            'observation_id': self.observation.id,
            'comment_id': self.comment.id
        })
        request = self.factory.patch(url, {'text': 'Updated'})
        request.user = AnonymousUser()
        request.DATA = {'review_status': 'resolved'}

        view = CommentAbstractAPIView()
        view.update_and_respond(request, self.comment)


class AllContributionsSingleCommentAPIViewTest(TestCase):
    def setUp(self):
        self.admin = UserF.create()
        self.creator = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.creator
        })

    def test_get_object_with_admin(self):
        view = AllContributionsSingleCommentAPIView()
        observation = view.get_object(
            self.admin, self.project.id, self.observation.id)
        self.assertEqual(observation, self.observation)

    def test_get_object_with_creator(self):
        view = AllContributionsSingleCommentAPIView()
        view.get_object(self.creator, self.project.id, self.observation.id)

    @raises(Project.DoesNotExist)
    def test_get_object_with_some_dude(self):
        some_dude = UserF.create()
        view = AllContributionsSingleCommentAPIView()
        view.get_object(some_dude, self.project.id, self.observation.id)


class GetProjectComments(APITestCase):
    def setUp(self):
        self.contributor = UserF.create()
        self.admin = UserF.create()
        self.non_member = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.contributor
        })
        comment = CommentFactory.create(**{
            'commentto': self.observation
        })
        response = CommentFactory.create(**{
            'commentto': self.observation,
            'respondsto': comment
        })
        CommentFactory.create(**{
            'commentto': self.observation,
            'respondsto': response
        })
        CommentFactory.create(**{
            'commentto': self.observation,
            'respondsto': comment
        })
        CommentFactory.create(**{
            'commentto': self.observation
        })

    def get_response(self, user):
        factory = APIRequestFactory()
        request = factory.get(
            '/api/projects/%s/observations/%s/comments/' %
            (self.project.id, self.observation.id)
        )
        force_authenticate(request, user=user)
        view = AllContributionsCommentsAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id
        ).render()

    def test_get_comments_with_admin(self):
        response = self.get_response(self.admin)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_comments_with_contributor(self):
        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_comments_with_non_member(self):
        response = self.get_response(self.non_member)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class AddCommentToPrivateProjectTest(APITestCase):
    def setUp(self):
        self.contributor = UserF.create()
        self.admin = UserF.create()
        self.non_member = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor]
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.contributor
        })

    def get_response(self, user):
        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (self.project.id, self.observation.id),
            {'text': 'A comment to the observation'}
        )
        force_authenticate(request, user=user)
        view = AllContributionsCommentsAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id
        ).render()

    def test_add_comment_to_observation_with_admin(self):
        response = self.get_response(self.admin)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_add_comment_to_observation_with_contributor(self):
        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_add_review_comment_to_observation_with_contributor(self):
        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (self.project.id, self.observation.id),
            {
                'text': 'A review comment to the observation',
                'review_status': 'open'
            }
        )
        force_authenticate(request, user=self.contributor)
        view = AllContributionsCommentsAPIView.as_view()
        response = view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'review'
        )

    def test_add_closed_review_comment_to_observation_with_contributor(self):
        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (self.project.id, self.observation.id),
            {
                'text': 'A review comment to the observation',
                'review_status': 'resolved'
            }
        )
        force_authenticate(request, user=self.contributor)
        view = AllContributionsCommentsAPIView.as_view()
        response = view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )

    def test_add_comment_to_observation_with_non_member(self):
        response = self.get_response(self.non_member)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_add_comment_to_draft(self):
        self.observation.status = 'draft'
        self.observation.save()

        response = self.get_response(self.observation.creator)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class AddCommentToPublicProjectTest(APITestCase):
    def setUp(self):
        self.contributor = UserF.create()
        self.admin = UserF.create()
        self.non_member = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{'isprivate': False}
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.contributor
        })

    def get_response(self, user):
        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/maps/all-contributions/%s/comments/' %
            (self.project.id, self.observation.id),
            {'text': 'A comment to the observation'}
        )
        force_authenticate(request, user=user)
        view = AllContributionsCommentsAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id
        ).render()

    def test_add_comment_to_observation_with_admin(self):
        response = self.get_response(self.admin)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_add_comment_to_observation_with_contributor(self):
        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_add_comment_to_observation_with_non_member(self):
        response = self.get_response(self.non_member)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_add_comment_to_observation_with_anonymous(self):
        response = self.get_response(AnonymousUser())
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class AddCommentToWrongProjectObservation(APITestCase):
    def test(self):
        admin = UserF.create()
        project = ProjectF(add_admins=[admin])
        observation = ObservationFactory.create()

        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (project.id, observation.id),
            {'text': 'A comment to the observation'}
        )
        force_authenticate(request, user=admin)
        view = AllContributionsCommentsAPIView.as_view()
        response = view(
            request,
            project_id=project.id,
            observation_id=observation.id
        ).render()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class AddResponseToProjectCommentTest(APITestCase):
    def test(self):
        admin = UserF.create()
        project = ProjectF(add_admins=[admin])
        observation = ObservationFactory.create(**{
            'project': project
        })
        comment = CommentFactory.create(**{
            'commentto': observation
        })

        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (project.id, observation.id),
            {
                'text': 'Response to a comment',
                'respondsto': comment.id
            }
        )
        force_authenticate(request, user=admin)
        view = AllContributionsCommentsAPIView.as_view()
        response = view(
            request,
            project_id=project.id,
            observation_id=observation.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(
            json.loads(response.content).get('respondsto'),
            comment.id
        )


class AddResponseToWrongProjectCommentTest(APITestCase):
    def test(self):
        admin = UserF.create()
        project = ProjectF(add_admins=[admin])
        observation = ObservationFactory.create(**{
            'project': project
        })
        comment = CommentFactory.create()

        factory = APIRequestFactory()
        request = factory.post(
            '/api/projects/%s/observations/%s/comments/' %
            (project.id, observation.id),
            {
                'text': 'Response to a comment',
                'respondsto': comment.id
            }
        )
        force_authenticate(request, user=admin)
        view = AllContributionsCommentsAPIView.as_view()
        response = view(
            request,
            project_id=project.id,
            observation_id=observation.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            json.loads(response.content).get('error'),
            'The comment you try to respond to is not a comment to the '
            'observation.'
        )


class DeleteProjectCommentTest(APITestCase):
    def setUp(self):
        self.contributor = UserF.create()
        self.admin = UserF.create()
        self.non_member = UserF.create()
        self.project = ProjectF(
            add_admins=[self.admin],
            add_contributors=[self.contributor],
            **{'isprivate': False}
        )
        self.observation = ObservationFactory.create(**{
            'project': self.project,
            'creator': self.contributor
        })
        self.comment = CommentFactory.create(**{
            'commentto': self.observation
        })
        self.comment_to_remove = CommentFactory.create(**{
            'commentto': self.observation,
            'creator': self.contributor
        })

    def get_response(self, user):
        factory = APIRequestFactory()
        request = factory.delete(
            '/api/projects/%s/observations/%s/comments/%s/' %
            (self.project.id, self.observation.id, self.comment_to_remove.id),
            {'text': 'A comment to the observation'}
        )
        force_authenticate(request, user=user)
        view = AllContributionsSingleCommentAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            observation_id=self.observation.id,
            comment_id=self.comment_to_remove.id
        ).render()

    def test_delete_comment_with_admin(self):
        response = self.get_response(self.admin)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertIn(self.comment, observation.comments.all())
        self.assertNotIn(self.comment_to_remove, observation.comments.all())

    def test_delete_comment_with_comment_creator(self):
        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertIn(self.comment, observation.comments.all())
        self.assertNotIn(self.comment_to_remove, observation.comments.all())

    def test_delete_review_comment_with_comment_creator(self):
        self.comment_to_remove.review_status = 'open'
        self.comment_to_remove.save()
        self.observation.status = 'review'
        self.observation.save()

        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(observation.status, 'active')

        self.assertIn(self.comment, observation.comments.all())
        self.assertNotIn(self.comment_to_remove, observation.comments.all())

    def test_delete_one_review_comment_with_comment_creator(self):
        self.comment.review_status = 'open'
        self.comment.save()
        self.comment_to_remove.review_status = 'open'
        self.comment_to_remove.save()
        self.observation.status = 'review'
        self.observation.save()

        response = self.get_response(self.contributor)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        observation = Observation.objects.get(pk=self.observation.id)
        self.assertEqual(observation.status, 'review')

        self.assertIn(self.comment, observation.comments.all())
        self.assertNotIn(self.comment_to_remove, observation.comments.all())

    def test_resolve_nested_comment_with_admin(self):
        self.comment.respondsto = self.comment_to_remove
        self.comment.review_status = 'open'
        self.comment.save()

        self.comment_to_remove.review_status = None
        self.comment_to_remove.save()

        self.observation.status = 'review'
        self.observation.save()

        response = self.get_response(self.admin)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            Observation.objects.get(pk=self.observation.id).status,
            'active'
        )


class DeleteWrongProjectComment(APITestCase):
    def test(self):
        admin = UserF.create()
        project = ProjectF(add_admins=[admin])
        observation = ObservationFactory.create(**{
            'project': project
        })
        comment = CommentFactory.create()

        factory = APIRequestFactory()
        request = factory.delete(
            '/api/projects/%s/observations/%s/comments/%s/' %
            (project.id, observation.id, comment.id),
            {'text': 'A comment to the observation'}
        )
        force_authenticate(request, user=admin)
        view = AllContributionsSingleCommentAPIView.as_view()
        response = view(
            request,
            project_id=project.id,
            observation_id=observation.id,
            comment_id=comment.id
        ).render()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



import datetime
import factory

from geokey.users.tests.model_factories import UserF

from ..models import Application


class ApplicationFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Application

    name = factory.Sequence(lambda n: 'name_%d' % n)
    description = factory.LazyAttribute(lambda o: '%s description' % o.name)
    created_at = datetime.date(2014, 11, 11)
    user = factory.SubFactory(UserF)
    download_url = 'http://example.com'
    redirect_uris = ['http://example.com/app']
    status = 'active'



import json
from io import BytesIO

from django.test import TestCase

from geokey.contributions.parsers.geojson import GeoJsonParser


class GeoJsonParserTest(TestCase):
    def test_parse_new_contribution_with_existing_location(self):
        obj = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.144415497779846,
                    51.54671869005856
                ]
            },
            "location": {
                "id": 1
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }

        ref = {
            "id": None,
            "location": {
                "id": 1,
                "geometry": '{"type": "Point","coordinates": [ '
                            '-0.144415497779846, 51.54671869005856]}',
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }
        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('id' in parsed)
        self.assertEqual(parsed.get('meta'), ref.get('meta'))
        self.assertEqual(
            parsed.get('location').get('id'),
            ref.get('location').get('id')
        )
        self.assertFalse('name' in parsed.get('location'))
        self.assertFalse('description' in parsed.get('location'))
        self.assertEqual(parsed.get('properties'), ref.get('properties'))

    def test_parse_new_contribution_with_new_empty_location(self):
        obj = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.144415497779846,
                    51.54671869005856
                ]
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }

        ref = {
            "id": None,
            "location": {
                "geometry": '{"type": "Point","coordinates": [ '
                            '-0.144415497779846, 51.54671869005856]}',
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }
        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('id' in parsed)
        self.assertEqual(parsed.get('meta'), ref.get('meta'))
        self.assertFalse('id' in parsed.get('location'))
        self.assertFalse('name' in parsed.get('location'))
        self.assertFalse('description' in parsed.get('location'))
        self.assertEqual(parsed.get('properties'), ref.get('properties'))

    def test_parse_new_contribution_with_new_full_location(self):
        obj = {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.144415497779846,
                    51.54671869005856
                ]
            },
            "location": {
                "name": "Location",
                "description": "Location description",
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }

        ref = {
            "id": None,
            "location": {
                "name": "Location",
                "description": "Location description",
                "geometry": '{"type": "Point","coordinates": [ '
                            '-0.144415497779846, 51.54671869005856]}',
            },
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            },
            "meta": {
                "category": 40,
                "status": "active"
            }
        }
        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('id' in parsed)
        self.assertEqual(parsed.get('meta'), ref.get('meta'))
        self.assertEqual(
            parsed.get('location').get('id'),
            ref.get('location').get('id')
        )
        self.assertEqual(
            parsed.get('location').get('name'),
            ref.get('location').get('name')
        )
        self.assertEqual(
            parsed.get('location').get('description'),
            ref.get('location').get('description')
        )
        self.assertEqual(parsed.get('properties'), ref.get('properties'))

    def test_parse_update_properties_only(self):
        obj = {
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            }
        }

        ref = {
            "properties": {
                "child_friendly": False,
                "name": "The Grafton",
                "address": "20 Prince of Wales Rd, London NW5 3LG"
            }
        }
        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('meta' in parsed)
        self.assertFalse('location' in parsed)
        self.assertEqual(parsed.get('properties'), ref.get('properties'))

    def test_parse_update_status_only(self):
        obj = {
            "id": 12,
            "meta": {
                "status": "review"
            }
        }

        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertEqual(parsed.get('id'), 12)
        self.assertEqual('review', parsed.get('meta').get('status'))
        self.assertFalse('location' in parsed)
        self.assertFalse('properties' in parsed)

    def test_parse_update_location_only(self):
        obj = {
            "location": {
                "id": 1,
                "name": "Location",
                "description": "Location description"
            }
        }

        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('id' in parsed)
        self.assertFalse('meta' in parsed)
        self.assertEqual(1, parsed.get('location').get('id'))
        self.assertEqual("Location", parsed.get('location').get('name'))
        self.assertEqual(
            "Location description",
            parsed.get('location').get('description')
        )
        self.assertFalse('properties' in parsed)

    def test_parse_update_geometry_only(self):
        obj = {
            "geometry": {
                "type": "Point",
                "coordinates": [
                    -0.144415497779846,
                    51.54671869005856
                ]
            },
        }

        parser = GeoJsonParser()
        parsed = parser.parse(BytesIO(json.dumps(obj)))

        self.assertFalse('id' in parsed)
        self.assertFalse('meta' in parsed)
        self.assertFalse('id' in parsed.get('location'))
        self.assertFalse('name' in parsed.get('location'))
        self.assertFalse('description' in parsed.get('location'))
        self.assertEqual(
            parsed.get('location').get('geometry'),
            '{"type": "Point", "coordinates": ['
            '-0.144415497779846, 51.54671869005856]}'
        )
        self.assertFalse('properties' in parsed)



from .base import *

# Set debug to True for development
DEBUG = True
TEMPLATE_DEBUG = DEBUG
LOGGING_OUTPUT_ENABLED = DEBUG
LOGGING_LOG_SQL = DEBUG

# Output email on the console
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Disable caching while in development
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
}

INSTALLED_APPS += (
    'django_nose',
)

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

# set up django-devserver if installed
try:
    import devserver
    INSTALLED_APPS += (
        'devserver',
    )
    # more details at https://github.com/dcramer/django-devserver#configuration
    DEVSERVER_DEFAULT_ADDR = '0.0.0.0'
    DEVSERVER_DEFAULT_PORT = '8000'
    DEVSERVER_AUTO_PROFILE = False  # use decorated functions
    DEVSERVER_TRUNCATE_SQL = True  # squash verbose output, show from/where
    DEVSERVER_MODULES = (
        # uncomment if you want to show every SQL executed
        # 'devserver.modules.sql.SQLRealTimeModule',
        # show sql query summary
        'devserver.modules.sql.SQLSummaryModule',
        # Total time to render a request
        'devserver.modules.profile.ProfileSummaryModule',

        # Modules not enabled by default
        # 'devserver.modules.ajax.AjaxDumpModule',
        # 'devserver.modules.profile.MemoryUseModule',
        # 'devserver.modules.cache.CacheSummaryModule',
        # see documentation for line profile decorator examples
        # 'devserver.modules.profile.LineProfilerModule',
        # show django session information
        # 'devserver.modules.request.SessionInfoModule',
    )

    EMAIL_HOST = 'localhost'
    EMAIL_PORT = 1025
    EMAIL_HOST_USER = ''
    EMAIL_HOST_PASSWORD = ''
    EMAIL_USE_TLS = False
    DEFAULT_FROM_EMAIL = 'testing@example.com'
except ImportError:
    pass



from django.views.generic import CreateView, TemplateView
from django.shortcuts import redirect
from django.core.exceptions import PermissionDenied
from django.contrib import messages
from django.utils.html import strip_tags
from django.contrib.gis.geos import GEOSGeometry

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from braces.views import LoginRequiredMixin

from geokey.core.decorators import (
    handle_exceptions_for_ajax, handle_exceptions_for_admin
)
from geokey.core.exceptions import Unauthenticated
from geokey.users.serializers import UserSerializer
from geokey.users.models import User, UserGroup
from geokey.categories.models import Category

from .base import STATUS
from .models import Project, Admins
from .forms import ProjectCreateForm
from .serializers import ProjectSerializer


# ############################################################################
#
# Administration views
#
# ############################################################################

class ProjectCreate(LoginRequiredMixin, CreateView):
    """
    Displays the create project page
    `/admin/projects/new`
    """
    form_class = ProjectCreateForm
    template_name = 'projects/project_create.html'

    def form_valid(self, form):
        """
        Creates the project and redirects to the project overview page

        Parameters
        ----------
        form : geokey.projects.forms.ProjectCreateForm
            Represents the user input
        """
        data = form.cleaned_data
        project = Project.create(
            strip_tags(data.get('name')),
            strip_tags(data.get('description')),
            data.get('isprivate'),
            data.get('everyone_contributes'),
            self.request.user
        )
        messages.success(self.request, "The project has been created.")
        return redirect('admin:project_overview', project_id=project.id)


class ProjectsInvolved(LoginRequiredMixin, TemplateView):
    """
    Displays a list of all projects the user is involved in
    """
    template_name = 'projects/projects_involved.html'

    def get_context_data(self):
        """
        Returns the context to render the view. Overwrites the method to add
        the list of projects to the context.

        Returns
        -------
        dict
            context
        """
        projects = Project.objects.get_list(self.request.user).exclude(
            admins=self.request.user)
        project_list = []

        for project in projects:
            project_list.append({
                'name': project.name,
                'role': project.get_role(self.request.user),
                'contributions': project.observations.filter(
                    creator=self.request.user).count(),
            })

        return {
            'projects': project_list
        }


class ProjectOverview(LoginRequiredMixin, TemplateView):
    """
    Displays the project overview page
    `/admin/projects/:project_id`
    """
    model = Project
    template_name = 'projects/project_overview.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the project, number of contributions and number of user contributions
        to the context.

        Parameters
        ----------
        project_id : int
            identifies the project in the database

        Returns
        -------
        dict
            context
        """
        user = self.request.user
        project = Project.objects.as_admin(user, project_id)
        contributions = project.observations.all()

        return {
            'project': project,
            'allcontributions': contributions.count(),
            'contributions': contributions.filter(
                creator=self.request.user).count()
        }


class ProjectExtend(LoginRequiredMixin, TemplateView):
    """
    Displays the page to edit the geograhic extent of the project
    """
    template_name = 'projects/project_extend.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the project to the context.

        Parameters
        ----------
        project_id : int
            identifies the project in the database

        Returns
        -------
        dict
            context
        """
        project = Project.objects.as_admin(self.request.user, project_id)

        context = super(ProjectExtend, self).get_context_data()
        context['project'] = project

        return context

    def post(self, request, project_id):
        """
        Adds or updates the geographic extent of the project.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        data = request.POST
        context = self.get_context_data(project_id)
        project = context.pop('project', None)
        geometry = data.get('geometry')

        if project is not None:
            if geometry is not None and len(geometry) > 0:
                project.geographic_extend = GEOSGeometry(data.get('geometry'))
            else:
                project.geographic_extend = None

            project.save()

            messages.success(
                self.request,
                'The geographic extent has been updated successfully.'
            )

            context['project'] = project
        return self.render_to_response(context)


class ProjectSettings(LoginRequiredMixin, TemplateView):
    """
    Displays the project settings page
    `/admin/projects/:project_id/settings/`
    """
    model = Project
    template_name = 'projects/project_settings.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id):
        """
        Returns the context to render the view. Overwrites the method to add
        the project and status types to the context.

        Parameters
        ----------
        project_id : int
            identifies the project in the database

        Returns
        -------
        dict
            context
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        return {
            'project': project,
            'status_types': STATUS
        }

    def post(self, request, project_id):
        """
        Updates the project settings

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template
        """
        context = self.get_context_data(project_id)
        project = context.pop('project')

        if project is not None:
            data = request.POST

            project.name = strip_tags(data.get('name'))
            project.description = strip_tags(data.get('description'))
            project.everyone_contributes = data.get('everyone_contributes')
            project.save()

            messages.success(self.request, "The project has been updated.")
            context['project'] = project
        return self.render_to_response(context)


class ProjectDelete(LoginRequiredMixin, TemplateView):
    """
    Deletes a project
    """
    template_name = 'base.html'

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, **kwargs):
        """
        Returns the context to render the view. Overwrites the method to add
        the project and status types to the context.

        Parameters
        ----------
        project_id : int
            identifies the project in the database

        Returns
        -------
        dict
            context
        """
        project = Project.objects.as_admin(self.request.user, project_id)
        return super(ProjectDelete, self).get_context_data(
            project=project, **kwargs)

    def get(self, request, project_id):
        """
        Deletes the project

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            redirecting to the dashboard

        django.http.HttpResponse
            If user is not administrator of the project, the error message is
            rendered.
        """
        context = self.get_context_data(project_id)
        project = context.pop('project', None)

        if project is not None:
            project.delete()

            messages.success(self.request, "The project has been deleted.")
            return redirect('admin:dashboard')

        return self.render_to_response(context)


# ############################################################################
#
# AJAX API views
#
# ############################################################################

class ProjectUpdate(APIView):
    """
    AJAX Endpoint for a project update.
    /ajax/projects/:project_id
    """

    @handle_exceptions_for_ajax
    def put(self, request, project_id):
        """
        Updates a project.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Response containing the serialised project or an error message
        """
        project = Project.objects.as_admin(request.user, project_id)
        serializer = ProjectSerializer(
            project, data=request.DATA, partial=True,
            fields=(
                'id', 'name', 'description', 'status', 'isprivate',
                'everyone_contributes'
            )
        )
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class ProjectAdmins(APIView):
    """
    AJAX Endpoint for project administrators.
    /ajax/projects/:project_id/admins
    """

    @handle_exceptions_for_ajax
    def post(self, request, project_id):
        """
        Adds a user to the admin group.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Response containing the serialised list of admins or an error
            message.
        """
        project = Project.objects.as_admin(request.user, project_id)
        user_id = request.DATA.get('userId')

        try:
            user = User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return Response(
                'The user you are trying to add to the user group does ' +
                'not exist',
                status=status.HTTP_400_BAD_REQUEST
            )

        Admins.objects.create(project=project, user=user)

        serializer = UserSerializer(project.admins.all(), many=True)
        return Response(
            {'users': serializer.data}, status=status.HTTP_201_CREATED)


class ProjectAdminsUser(APIView):
    """
    AJAX Endpoint for a single project administrator.
    /ajax/projects/:project_id/admins
    """

    @handle_exceptions_for_ajax
    def delete(self, request, project_id, user_id):
        """
        Removes a user from the user group.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Empty response if successful or response containing an error
            message.
        """
        project = Project.objects.as_admin(request.user, project_id)
        user = project.admins.get(pk=user_id)
        Admins.objects.get(project=project, user=user).delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


class CategoriesReorderView(APIView):
    """
    AJAX Endpoint to re-order categories in a project.
    /ajax/projects/:project_id/cotegories/re-order
    """
    @handle_exceptions_for_ajax
    def post(self, request, project_id):
        """
        Reorders the cateories in the project.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Contains the serialised project or an error message
        """
        project = Project.objects.as_admin(request.user, project_id)

        try:
            project.re_order_categories(request.DATA.get('order'))

            serializer = ProjectSerializer(
                project,
                fields=(
                    'id', 'name', 'description', 'status', 'isprivate',
                    'everyone_contributes'
                )
            )
            return Response(serializer.data)
        except Category.DoesNotExist:
            return Response(
                {'error': 'One or more categories ids where not found.'},
                status=status.HTTP_400_BAD_REQUEST
            )


# ############################################################################
#
# Public API views
#
# ############################################################################

class Projects(APIView):
    """
    API Endpoint for project list in the public API.
    /api/projects/
    """
    @handle_exceptions_for_ajax
    def get(self, request):
        """
        Returns a list a all projects accessible to the user.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.

        Returns
        -------
        rest_framework.reponse.Response
            Contains serialised list of projects
        """
        projects = Project.objects.get_list(
            request.user).filter(status='active')
        serializer = ProjectSerializer(
            projects, many=True, context={'user': request.user},
            fields=('id', 'name', 'description', 'user_info')
        )
        return Response(serializer.data)


class SingleProject(APIView):
    """
    API Endpoint for single project in the public API.
    /api/projects/:project_id/
    """
    @handle_exceptions_for_ajax
    def get(self, request, project_id):
        """
        Returns a single project.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Contains the serialised project

        Raises
        ------
        PermissionDenied
            if the project is inactive, is handled in the
            handle_exceptions_for_ajax decorator
        """
        project = Project.objects.get_single(request.user, project_id)
        if project.status == 'active':
            serializer = ProjectSerializer(
                project, context={'user': request.user}
            )
            return Response(serializer.data)

        raise PermissionDenied('The project is inactive and therefore '
                               'not accessable through the public API.')


class ProjectContactAdmins(APIView):
    """
    API Endpoint for single project in the public API.
    /api/projects/:project_id/get-in-touch/
    """
    @handle_exceptions_for_ajax
    def post(self, request, project_id):
        """
        Sends an email to all admins that are contact persons for the given
        project.

        Parameter
        ---------
        request : rest_framework.request.Request
            Object representing the request.
        project_id : int
            identifies the project in the database

        Returns
        -------
        rest_framework.reponse.Response
            Empty reponse indicating success

        Raises
        ------
        Unauthenticated
            if the user is anonymous; is handled in the
            handle_exceptions_for_ajax decorator
        """
        user = request.user
        if user.is_anonymous():
            raise Unauthenticated('Unauthenticated users can not contact the '
                                  'administrators of the project.')

        email_text = self.request.DATA.get('email_text')
        project = Project.objects.get_single(request.user, project_id)

        project.contact_admins(user, email_text)
        return Response(status=status.HTTP_204_NO_CONTENT)



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.http import HttpResponseRedirect
from django.contrib.sites.models import Site

from rest_framework.test import APIRequestFactory, force_authenticate

from geokey.users.tests.model_factories import UserF
from geokey.users.models import User
from geokey.projects.tests.model_factories import ProjectF

from ..views import (
    PlatformSettings,
    ProjectsList,
    ManageSuperUsers,
    AddSuperUsersAjaxView,
    DeleteSuperUsersAjaxView
)


class SiteSettingsTest(TestCase):
    def test_get_context_data(self):
        view = PlatformSettings()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().get(url)
        view.request = request
        context = view.get_context_data()
        self.assertIsNotNone(context.get('site'))

    def test_get_with_superuser(self):
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': True})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_user(self):
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': False})
        view.request = request
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Superuser tools are for superusers only. You are not a superuser.'
        )

    def test_get_with_anonymous(self):
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        view.request = request
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))

    def test_post_with_superuser(self):
        data = {
            'name': 'New Name',
            'domain': 'http://domain'
        }
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().post(url, data)
        request.user = UserF.create(**{'is_superuser': True})

        from django.contrib.messages.storage.fallback import FallbackStorage
        setattr(request, 'session', 'session')
        messages = FallbackStorage(request)
        setattr(request, '_messages', messages)

        response = view(request).render()
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response, 'The platform settings have been updated.')

        site = Site.objects.get(pk=1)
        self.assertEqual(site.name, data.get('name'))
        self.assertEqual(site.domain, data.get('domain'))

    def test_post_with_user(self):
        data = {
            'name': 'New Name',
            'domain': 'http://domain'
        }
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().post(url, data)
        request.user = UserF.create(**{'is_superuser': False})
        view.request = request
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Superuser tools are for superusers only. You are not a superuser.'
        )

    def test_post_with_anonymous(self):
        data = {
            'name': 'New Name',
            'domain': 'http://domain'
        }
        view = PlatformSettings.as_view()
        url = reverse('admin:superuser_index')
        request = APIRequestFactory().post(url, data)
        request.user = AnonymousUser()
        view.request = request
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))


class ProjectsListTest(TestCase):
    def test_get_context_data(self):
        user = UserF.create(**{'is_superuser': True})
        ProjectF.create_batch(5, add_admins=[user])
        ProjectF.create_batch(5)
        view = ProjectsList()
        context = view.get_context_data()
        self.assertEqual(len(context.get('projects')), 10)

    def test_get_with_superuser(self):
        view = ProjectsList.as_view()
        url = reverse('admin:superuser_projects')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': True})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_user(self):
        view = ProjectsList.as_view()
        url = reverse('admin:superuser_projects')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': False})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Superuser tools are for superusers only. You are not a superuser.'
        )

    def test_get_with_anonymous(self):
        view = ProjectsList.as_view()
        url = reverse('admin:superuser_projects')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))


class ManageSuperUsersTest(TestCase):
    def test_get_context_data(self):
        UserF.create_batch(2, **{'is_superuser': True})
        UserF.create_batch(2, **{'is_superuser': False})
        view = ManageSuperUsers()
        context = view.get_context_data()
        self.assertEqual(len(context.get('superusers')), 2)

    def test_get_with_superuser(self):
        view = ManageSuperUsers.as_view()
        url = reverse('admin:superuser_manage_users')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': True})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)

    def test_get_with_user(self):
        view = ManageSuperUsers.as_view()
        url = reverse('admin:superuser_manage_users')
        request = APIRequestFactory().get(url)
        request.user = UserF.create(**{'is_superuser': False})
        response = view(request).render()
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            'Superuser tools are for superusers only. You are not a superuser.'
        )

    def test_get_with_anonymous(self):
        view = ManageSuperUsers.as_view()
        url = reverse('admin:superuser_manage_users')
        request = APIRequestFactory().get(url)
        request.user = AnonymousUser()
        response = view(request)
        self.assertTrue(isinstance(response, HttpResponseRedirect))


class AddSuperUsersAjaxViewTest(TestCase):
    def test_post_with_superuser(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': True})
        user_to_add = UserF.create(**{'is_superuser': False})
        request = factory.post(
            reverse('ajax:superusers_adduser'),
            {'userId': user_to_add.id}
        )
        force_authenticate(request, user=user)
        view = AddSuperUsersAjaxView.as_view()
        response = view(request).render()

        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 2)

    def test_post_with_user(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': False})
        user_to_add = UserF.create(**{'is_superuser': False})
        request = factory.post(
            reverse('ajax:superusers_adduser'),
            {'userId': user_to_add.id}
        )
        force_authenticate(request, user=user)
        view = AddSuperUsersAjaxView.as_view()
        response = view(request).render()

        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 0)

    def test_post_non_existing_user(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': True})
        request = factory.post(
            reverse('ajax:superusers_adduser'),
            {'userId': 78463857934859}
        )
        force_authenticate(request, user=user)
        view = AddSuperUsersAjaxView.as_view()
        response = view(request).render()

        self.assertEqual(response.status_code, 400)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 1)


class DeleteSuperUsersAjaxViewTest(TestCase):
    def test_delete_with_superuser(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': True})
        user_to_remove = UserF.create(**{'is_superuser': True})
        request = factory.delete(
            reverse('ajax:superusers_deleteuser', kwargs={
                'user_id': user_to_remove.id
            })
        )
        force_authenticate(request, user=user)
        view = DeleteSuperUsersAjaxView.as_view()
        response = view(request, user_id=user_to_remove.id).render()

        self.assertEqual(response.status_code, 204)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 1)

    def test_delete_with_user(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': False})
        user_to_remove = UserF.create(**{'is_superuser': True})
        request = factory.delete(
            reverse('ajax:superusers_deleteuser', kwargs={
                'user_id': user_to_remove.id
            })
        )
        force_authenticate(request, user=user)
        view = DeleteSuperUsersAjaxView.as_view()
        response = view(request, user_id=user_to_remove.id).render()

        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 1)

    def test_delete_not_existing_user(self):
        factory = APIRequestFactory()
        user = UserF.create(**{'is_superuser': True})
        request = factory.delete(
            reverse('ajax:superusers_deleteuser', kwargs={
                'user_id': 84774358734
            })
        )
        force_authenticate(request, user=user)
        view = DeleteSuperUsersAjaxView.as_view()
        response = view(request, user_id=84774358734).render()

        self.assertEqual(response.status_code, 404)
        self.assertEqual(len(User.objects.filter(is_superuser=True)), 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='observation',
            name='creator',
            field=models.ForeignKey(related_name='creator', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='observation',
            name='location',
            field=models.ForeignKey(related_name='locations', to='contributions.Location'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='observation',
            name='project',
            field=models.ForeignKey(related_name='observations', to='projects.Project'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='observation',
            name='updator',
            field=models.ForeignKey(related_name='updator', to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mediafile',
            name='contribution',
            field=models.ForeignKey(related_name='files_attached', to='contributions.Observation'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mediafile',
            name='creator',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='location',
            name='creator',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='location',
            name='private_for_project',
            field=models.ForeignKey(to='projects.Project', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='historicalobservation',
            name='history_user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='comment',
            name='commentto',
            field=models.ForeignKey(related_name='comments', to='contributions.Observation'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='comment',
            name='creator',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='comment',
            name='respondsto',
            field=models.ForeignKey(related_name='responses', blank=True, to='contributions.Comment', null=True),
            preserve_default=True,
        ),
    ]



from django import forms

from .models import Application
from django.utils.html import strip_tags


class AppCreateForm(forms.ModelForm):
    """
    Validates the inputs against the model definition.
    Used in .views.AppCreateView
    """
    class Meta:
        model = Application
        fields = ('name', 'description', 'download_url', 'redirect_uris',
                  'authorization_grant_type')

    def clean(self):
        """
        Overwrites ModelForm's clean method to strip HTML Tags from name and
        description

        Returns
        -------
        dict
            Cleaned form data including HTML free name and description
        """
        cleaned_data = super(AppCreateForm, self).clean()
        cleaned_data['name'] = strip_tags(cleaned_data['name'])
        cleaned_data['description'] = strip_tags(cleaned_data['description'])

        return cleaned_data



from django.conf.urls import patterns, url

from geokey.projects import views as project_views
from geokey.categories import views as category_views
from geokey.users import views as login_views
from geokey.applications import views as app_views
from geokey.superusertools import views as superuser


urlpatterns = patterns(
    '',
    url(r'^$', login_views.Index.as_view(), name='index'),
    url(r'^dashboard/$', login_views.Dashboard.as_view(), name='dashboard'),

    # ###########################
    # PROJECTS
    # ###########################
    url(
        r'^projects/new/$',
        project_views.ProjectCreate.as_view(),
        name='project_create'),
    url(
        r'^projects/involved/$',
        project_views.ProjectsInvolved.as_view(),
        name='projects_involved'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/$',
        project_views.ProjectOverview.as_view(),
        name='project_overview'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/geographic-extent/$',
        project_views.ProjectExtend.as_view(),
        name='project_extend'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/settings/$',
        project_views.ProjectSettings.as_view(),
        name='project_settings'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/delete/$',
        project_views.ProjectDelete.as_view(),
        name='project_delete'),


    # ###########################
    # USER GROUPS
    # ###########################
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/$',
        login_views.UserGroupList.as_view(),
        name='usergroup_list'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/new/$',
        login_views.UserGroupCreate.as_view(),
        name='usergroup_create'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/(?P<group_id>[0-9]+)/$',
        login_views.UserGroupOverview.as_view(),
        name='usergroup_overview'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/administrators/$',
        login_views.AdministratorsOverview.as_view(),
        name='admins_overview'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/(?P<group_id>[0-9]+)/permissions/$',
        login_views.UserGroupPermissions.as_view(),
        name='usergroup_permissions'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/(?P<group_id>[0-9]+)/settings/$',
        login_views.UserGroupSettings.as_view(),
        name='usergroup_settings'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/(?P<group_id>[0-9]+)/delete/$',
        login_views.UserGroupDelete.as_view(),
        name='usergroup_delete'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/usergroups/(?P<group_id>[0-9]+)/data/$',
        login_views.UserGroupData.as_view(),
        name='usergroup_data'),

    # ###########################
    # CATEGORIES & FIELDS
    # ###########################
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/$',
        category_views.CategoryList.as_view(),
        name='category_list'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/new/$',
        category_views.CategoryCreate.as_view(),
        name='category_create'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/$',
        category_views.CategoryOverview.as_view(),
        name='category_overview'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/display/$',
        category_views.CategoryDisplay.as_view(),
        name='category_display'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/settings/$',
        category_views.CategorySettings.as_view(),
        name='category_settings'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/delete/$',
        category_views.CategoryDelete.as_view(),
        name='category_delete'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/fields/new/$',
        category_views.FieldCreate.as_view(),
        name='category_field_create'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/fields/(?P<field_id>[0-9]+)/$',
        category_views.FieldSettings.as_view(),
        name='category_field_settings'),
    url(
        r'^projects/(?P<project_id>[0-9]+)/categories/(?P<category_id>[0-9]+)/fields/(?P<field_id>[0-9]+)/delete/$',
        category_views.FieldDelete.as_view(),
        name='category_field_delete'),

    # ###########################
    # APPS
    # ###########################

    url(
        r'^apps/$',
        app_views.ApplicationOverview.as_view(),
        name='app_overview'),
    url(
        r'^apps/connected/$',
        app_views.ApplicationConnected.as_view(),
        name='app_connected'),
    url(
        r'^apps/register/$',
        app_views.ApplicationCreate.as_view(),
        name='app_register'),
    url(
        r'^apps/(?P<app_id>[0-9]+)/$',
        app_views.ApplicationSettings.as_view(),
        name='app_settings'),
    url(
        r'^apps/(?P<app_id>[0-9]+)/delete/$',
        app_views.ApplicationDelete.as_view(),
        name='app_delete'),
    url(
        r'^apps/(?P<app_id>[0-9]+)/disconnect/$',
        app_views.ApplicationDisconnect.as_view(),
        name='app_disconnect'),

    # ###########################
    # USERS
    # ###########################
    url(r'^profile/$',
        login_views.UserProfile.as_view(),
        name="userprofile"),
    url(r'^profile/notifications/$',
        login_views.UserNotifications.as_view(),
        name="notifications"),

    # ###########################
    # SUPER-USER TOOLS
    # ###########################
    url(r'^superuser-tools/$',
        superuser.PlatformSettings.as_view(),
        name="superuser_index"),
    url(r'^superuser-tools/projects/$',
        superuser.ProjectsList.as_view(),
        name="superuser_projects"),
    url(r'^superuser-tools/manage-users/$',
        superuser.ManageSuperUsers.as_view(),
        name="superuser_manage_users"),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def update_search_matches(apps, schema_editor):
    Observation = apps.get_model("contributions", "Observation")
    for observation in Observation.objects.all():
        observation.update_search_matches()
        observation.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0004_auto_20150121_1455'),
    ]

    operations = [
        migrations.RunPython(update_search_matches),
    ]
